package com.arubanetworks.meridiansamples

import android.content.Context
import android.graphics.Bitmap
import android.graphics.BitmapFactory
import android.graphics.drawable.Drawable
import android.os.Bundle
import android.webkit.URLUtil
import com.arubanetworks.meridian.editor.EditorKey
import com.arubanetworks.meridian.internal.util.Dev
import com.arubanetworks.meridian.maps.MapFragment
import com.arubanetworks.meridian.maps.Marker
import com.arubanetworks.meridian.maps.Transaction
import com.arubanetworks.meridian.tags.TagBeacon
import com.arubanetworks.meridian.tags.TagMarker
import com.arubanetworks.meridian.tags.TagStream
import com.arubanetworks.meridiansamples.LocationSharingMapFragment.Companion.getFormattedAgeString
import com.bumptech.glide.Glide
import com.bumptech.glide.load.resource.bitmap.CircleCrop
import com.bumptech.glide.request.target.CustomTarget
import com.bumptech.glide.request.transition.Transition


/**
 * A Fragment to demonstrate the use of the TagStream for use in displaying Tags.  Demonstrates how to
 * show all tags for a map or a single tag in a location
 */
class TagsFragment : MapFragment(), TagStream.Listener {

    // A collection to maintain our TagMarkers
    private val tagMarkers = HashMap<TagBeacon, SamplesTagMarker>()

    // The TagStream itself
    private var tagStream: TagStream? = null
    private var tagMac : String? = null
    private var selectedTagMarker: SamplesTagMarker? = null

    override fun onMapLoadStart() {
        super.onMapLoadStart()
        // Close out any existing tags when loading a new map key
        tagStream?.stopUpdatingTags()

        // Clear any references to old markers so we don't re-add them to the new map on accident
        tagMarkers.clear()

        // Create our TagStream and start it.
        tagStream = tagMac?.let {
            TagStream.Builder()
                    .addTagMac(Application.APP_KEY, it)
                    .setListener(this)
                    .build()
        } ?: run {
            mapView?.mapKey?.let {
                TagStream.Builder()
                        .addMapKey(it)
                        .setListener(this)
                        .build()
            }
        }
    }

    override fun onMarkerSelect(marker: Marker): Boolean {
        selectedTagMarker = null
        if (marker is SamplesTagMarker && tagMarkers.containsValue(marker)) {
            selectedTagMarker = marker
        }
        return false
    }

    override fun onMarkerDeselect(marker: Marker): Boolean {
        if (marker is SamplesTagMarker) {
            selectedTagMarker = null
        }
        return false
    }

    override fun markerForSelectedMarker(marker: Marker): Marker? {
        if (context != null) {
            if (marker is SamplesTagMarker) {
                val newMarker = SamplesTagMarker(context, marker.tagBeacon, true)
                newMarker.setPosition(marker.getPosition()[0], marker.getPosition()[1])
                // update last heard field (in marker details)
                val ageInMillis = System.currentTimeMillis() - marker.tagBeacon.location.timestamp.time
                if (context != null) {
                    selectedTagMarker!!.details = getString(R.string.last_heard) + ": " + getFormattedAgeString(requireContext().resources, ageInMillis)
                }
                selectedTagMarker = newMarker
                return newMarker
            }
        }
        return null
    }

    override fun onTagsUpdated(tags: List<TagBeacon>) {

        val transaction = Transaction.Builder().setType(Transaction.Type.UPDATE)

        // Iterate through the new or updated tags and add them to the map.
        for (tag in tags) {
            if (tagMac != null && !tagMac.equals(tag.mac, ignoreCase = true)) continue
            // ignore tags that are not on the current map
            if (mapView != null && mapView.mapKey != null && tag.mapId != mapView.mapKey.id) continue
            var currentMarker: SamplesTagMarker? = tagMarkers[tag]

            // check for a new Tag and add
            if (currentMarker == null) {
                if (context == null) {
                    return
                }
                currentMarker = SamplesTagMarker(requireContext(), tag, false)
                tagMarkers[tag] = currentMarker
            } else {
                // Otherwise update our current marker to reflect the new tag state
                val point = tag.location?.point
                point?.let { currentMarker.setPosition(it.x, it.y) }
                if (selectedTagMarker != null && selectedTagMarker!!.tagBeacon.mac.equals(tag.mac, ignoreCase = true)) {
                    selectedTagMarker!!.setPosition(tag.location.point.x, tag.location.point.y)
                    transaction.addMarker(selectedTagMarker!!).build()
                }
            }
            transaction.addMarker(currentMarker)
        }

        // Commit the transaction
        if (transaction.size > 0)
            mapView.commitTransaction(transaction.build())
    }

    override fun onTagsRemoved(tags: List<TagBeacon>) {
        val transaction = Transaction.Builder().setType(Transaction.Type.REMOVE)

        for (tag in tags) {
            if (tagMac != null && !tagMac.equals(tag.mac, ignoreCase = true)) continue
            // We can ignore removing the tags marker if we never added it.
            val currentMarker = tagMarkers[tag] ?: continue
            transaction.addMarker(currentMarker)
        }

        mapView?.commitTransaction(transaction.build())
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        tagMac = arguments?.getString(TAG_MAC)
    }

    override fun onPause() {
        super.onPause()
        // If we are paused we stop listening to tag updates
        tagStream?.stopUpdatingTags()
    }

    override fun onResume() {
        super.onResume()
        // If we are resuming, start listening for tags again.
        tagStream?.startUpdatingTags()
/*
        // example how to scan for tags that are nearby.
        val ts = TagScan()
        ts.findNearbyTags(
                Application.APP_KEY,
                null, // return all tags
                0, // use default scan length (12s)
                0, // don't filter by RSSI
                MeridianRequest.Listener<List<TagBeacon?>?> { tagBeacons ->
                    if (tagBeacons != null) {
                        Log.d("TagsFragment", "Found " + tagBeacons.size + " nearby tags")
                        for (tag in tagBeacons) {
                            Log.d("TagsFragment", "Found:" + tag.toString())
                        }
                    }
                },
                null
        )*/
    }

    override fun onDestroy() {
        super.onDestroy()
        // Clean up memory.
        tagStream?.dispose()
    }

    companion object {
        private const val TAG_MAC = "meridian.TAG_MAC"
        fun newInstance(mapKey: EditorKey): TagsFragment {
            val f = TagsFragment()
            f.arguments = Builder().setMapKey(mapKey).build().arguments
            return f
        }
        fun newInstance(mapKey: EditorKey, tagMac: String): TagsFragment {
            val f = TagsFragment()
            val args = Builder().setMapKey(mapKey).build().arguments
            if (args != null) {
                args.putString(TAG_MAC, tagMac)
                f.arguments = args
            }
            return f
        }
    }

    private inner class SamplesTagMarker (context: Context?, private val tag: TagBeacon, selected: Boolean) :
            TagMarker(context!!, tag) {
        private val context: Context? = context?.applicationContext
        private var bitmap: Bitmap? = null
        private val selected // if the tag marker is selected, this could be used to change the bitmap size for example
                : Boolean

        init {
            this.selected = selected
            weight = 1.1f
            name = tag.name
            val ageInMillis = System.currentTimeMillis() - tag.location.timestamp.time
            if (this.context != null) {
                details = getString(R.string.last_heard) + ": " + getFormattedAgeString(this.context.resources, ageInMillis)
            }
        }

        override fun getCollision(): Boolean {
            return false
        }

        override fun canBeSelected(): Boolean {
            return !selected
        }

        override fun getBitmap(): Bitmap? {
            if (bitmap != null) {
                return bitmap
            }
            val SIZE_AVATAR_DP = 40.0f
            if (activity != null) {
                val size = Dev.get().dpToPix(if (selected) (SIZE_AVATAR_DP * 1.5f) else SIZE_AVATAR_DP)
                if (URLUtil.isValidUrl(tag.imageURL)) {
                    activity!!.runOnUiThread(Runnable {
                        Glide.with(activity!!.applicationContext)
                                .asBitmap()
                                .load(tag.imageURL)
                                .transform(CircleCrop())
                                .into(object : CustomTarget<Bitmap?>(size, size) {
                                    override fun onResourceReady(bitmap: Bitmap, transition: Transition<in Bitmap?>?) {
                                        this@SamplesTagMarker.bitmap = bitmap
                                        this@SamplesTagMarker.invalidate(true)
                                    }

                                    override fun onLoadCleared(placeholder: Drawable?) {
                                        //do nothing
                                    }
                                })
                    })
                } else {
                    activity!!.runOnUiThread {
                        val bitmap1 = BitmapFactory.decodeResource(resources, R.drawable.mr_ic_tag)
                        bitmap = Bitmap.createScaledBitmap(bitmap1, size, size, true)
                        this@SamplesTagMarker.invalidate(true)
                    }
                }
            }
            // still loading
            return null
        }
    }
}
